#include "pmc_e1_if.h"

extern int board;
extern int chan_num;
extern char conf_file[256];

void ecfg_read_status (void);
void ecfg_factory_cfg (void);
void ecfg_restore_cfg (void);
void ecfg_save_cfg (void);

void main_menu_call (void);
void config_menu_call (void);
void c_framing_menu_call (void);
void c_multiframe_menu_call (void);
void test_menu_call (void);
void c_m_cas_menu_call (void);
unsigned char tsnum (void);

struct pmc_config_chan cfg;
struct pmc_err_ins inserr;
struct pmc_status_chan status;
struct pmc_cas_xmit_data abcd;
struct pmc_version version;

const code char * code off_on[] = {"Off", "On "}; 
const code char * code on_off[] = {"On ", "Off"};
const code char * code framing_transparent[] = {"Framing", "Transparent"};
const code char * code ami_hdb3[] = {"AMI", "HDB3"};
const code char * code line_int[] = {"Line", "Internal"};
const code char * code empty_empty[] = {"", ""};

const code char * code auto_off_on[] = {"Auto", "Off ", "On  "};
const code char * code auto_on_off[] = {"Auto", "On ", "Off  "};

void print_tristate (const code char * code msgs[], unsigned char val,
		     unsigned char mask_hi, unsigned char mask_lo,
		     unsigned char skip) {
    unsigned char off;

    off = (val & mask_hi) ? 0x2 : 0x0;
    off |= (val & mask_lo) ? 0x1 : 0x0;
    if (off == skip) {
        PRINTF ("bad value %d", off);
        return;
    }
    off %= 4;
    if (off >= skip) --off;
    off %= 4;
    PRINTF ("%s", msgs[off]);
}

void toggle_tristate (unsigned char *val, unsigned char mask_hi,
		      unsigned char mask_lo, unsigned char skip) {
    unsigned char off;

    off = (*val & mask_hi) ? 0x2 : 0x0;
    off |= (*val & mask_lo) ? 0x1 : 0x0;
    ++off;
    off %= 4;
    if (off == skip) ++off;
    off %= 4;
    *val &= ~(mask_hi | mask_lo);
    if (off & 0x1) *val |= mask_lo;
    if (off & 0x2) *val |= mask_hi;
}

const code char *_GET_VAR_STR (const code char **arr, unsigned long var, unsigned char bitmask) {
    if (var&bitmask)
	return arr[1];
    return (arr[0]);
}

#define PRINT_VAR(arr,var,bitmask) \
    PUTS_CODE (_GET_VAR_STR (arr,var,bitmask))

unsigned char get1bhex (void) reentrant {
    unsigned char c= MENUS_GETCH ();

    if (isdigit(c)) return c-'0';
    else return c-'a'+10;
}

unsigned char get2bhex () reentrant {
    unsigned char res;

    res = ((get1bhex ()&0x0f) << 4);
    res |= (get1bhex ()&0x0f);
    return res;
}

void ecfg_print_title (void) {
    PRINTF (DEV_TITLE " monitor v.%d.%02d %s"
	    " Updates: http://parabel.ru/" NL, VERSION_MAJOR, VERSION_MINOR, RELEASE_DATE);
    PRINTF ("PMC/chan=%d/%d, conf. file=\"%s\"" NL,
	    board, chan_num, conf_file);
}

void print_one_if_bit_set (unsigned char reg, unsigned char mask) {
    if ((reg & mask) == mask) PRINTF ("1");
    else PRINTF ("0");
}

void ecfg_print_head (void) {
    int i;
    unsigned long ver;

    ecfg_read_status ();
    ecfg_print_title ();
    ver = version.hw_fw_rev;
    PRINTF ("HW/FW/REV version=%lx/%lx/%lx, driver verision=%s" NL,
	    (ver>>8)&0xFF, (ver>>0)&0xFF, (ver>>16)&0xFF,
	    version.driver_version);

    PUTS_CODE (NL "Line status: ");
    PUTS_CODE ("LOS=");
    PRINT_VAR (off_on, status.stat0, STAT0_LOS);
    PUTS_CODE (", AIS=");
    PRINT_VAR (off_on, status.stat0, STAT0_AIS);

    PUTS_CODE (NL "Frame status: ");
    PUTS_CODE ("LOF=");
    PRINT_VAR (off_on, status.stat0, STAT0_LOF);
    PUTS_CODE (", Sa4..8=");
    print_one_if_bit_set (status.stat1, STAT1_NAT0);
    print_one_if_bit_set (status.stat1, STAT1_NAT1);
    print_one_if_bit_set (status.stat1, STAT1_NAT2);
    print_one_if_bit_set (status.stat1, STAT1_NAT3);
    print_one_if_bit_set (status.stat1, STAT1_NAT4);
#ifndef QUASAR_MINI
    PUTS_CODE (", Si0..1=");
    print_one_if_bit_set (status.stat1, STAT1_INAT0);
    print_one_if_bit_set (status.stat1, STAT1_INAT1);
#endif //QUASAR_MINI
    PUTS_CODE (", RAIS=");
    PRINT_VAR (off_on, status.stat0, STAT0_RAIS);

    PUTS_CODE (NL "CAS Multiframe: ");
    PUTS_CODE ("CAS LOM=");
    PRINT_VAR (off_on, status.stat0, STAT0_CAS_LOM);
    PUTS_CODE (", XYXX=");
    print_one_if_bit_set (status.stat2, STAT2_X1);
    print_one_if_bit_set (status.stat2, STAT2_Y);
    print_one_if_bit_set (status.stat2, STAT2_X2);
    print_one_if_bit_set (status.stat2, STAT2_X3);

    PUTS_CODE (NL "CRC4 Multiframe: ");
    PUTS_CODE ("CRC4 err=");
    PRINT_VAR (off_on, status.stat0, STAT0_CRC4);
    PUTS_CODE (", LOC=");
    PRINT_VAR (off_on, status.stat0, STAT0_CRC4_SYNC);
    PUTS_CODE (", E bit=");
    PRINT_VAR (off_on, status.stat0, STAT0_E_BIT);
    PUTS_CODE (NL);

    PRINTF ("Err counters: HDB3=%d, FAS=%d, CRC4=%d" NL,
	    status.hdb3, status.fas, status.crc4);
    if (!(status.stat0 & STAT0_CAS_LOM)) {
	PUTS_CODE ("ABCD status: ");
	for (i=0; i<16; ++i) {
            if (i && (i % 4 == 0)) PUTS_CODE (" ");
	    PRINTF ("%x%x", status.cas_data[i] >> 4,
		    status.cas_data[i] & 0x0F);
	}
    }
    PUTS_CODE_NL (NL);
}

/* === /Test/Loop menu === */
INST_MLT(line_t_l_lloop, 1, "LLOOP", off_on,
	 &(cfg.cfg4), CFG4_LLOOP);
INST_MLT(line_t_l_rloop, 2, "RLOOP", off_on,
	 &(cfg.cfg4), CFG4_RLOOP);
INST_MLT(line_t_l_ploop, 3, "PLOOP", off_on,
	 &(cfg.cfg4), CFG4_PLOOP);
const code void * const code t_loop_lines[] = {
    &line_t_l_lloop,
    &line_t_l_rloop,
#ifndef QUASAR_MINI
    &line_t_l_ploop,
#endif //QUASAR_MINI
    0
};
INST_MENU_LEVEL(t_loop_menu,test_menu_call,t_loop_lines,
		"Test/", "Loopback", 0);
void t_loop_menu_call (void) {
    curr_menu = &t_loop_menu;
}



/* === /Test menu === */
INST_MLS(line_t_loop, 1, "Loopback", &t_loop_menu);
INST_MLT(line_t_lof, 2, "Emu single LOF", empty_empty,
	 &(inserr.pmc_err), ERR_EMU0_LOF_FAS);
INST_MLT(line_t_loc, 3, "Emu single LOC(CRC4)", empty_empty,
	 &(inserr.pmc_err), ERR_EMU0_CRC4);
INST_MLT(line_t_los, 4, "LOS", off_on,
	 &(cfg.cfg4), CFG4_TX_DIS);
INST_MLT(line_t_ais, 5, "AIS", on_off, /*reverse*/
	 &(cfg.cfg4), CFG4_AIS);
const code void * const code test_lines[] = {
    &line_t_loop,
#ifndef QUASAR_MINI
    &line_t_lof,
    &line_t_loc,
    &line_t_los,
#endif //QUASAR_MINI
    &line_t_ais,
    0
};
INST_MENU_LEVEL(test_menu,main_menu_call,test_lines,
		"Test", "", 0);
void test_menu_call (void) {
    curr_menu = &test_menu;
}


/* === /Configuration/Framing/Multiframing/CRC4 menu === */
INST_MLT(line_c_m_crc4_tr, 1, "CRC4 transparent regeneration", off_on,
	 &(cfg.cfg3), CFG3_CRC4_TR);
INST_MLT(line_c_m_crc4_mode, 2, "CRC4 multiframe", on_off,
	 &(cfg.cfg3), CFG3_CRC4_DIS);
INST_MLT(line_c_m_crc4_acrc, 3, "Automatic CRC-interworking", on_off,
	 &(cfg.cfg3), CFG3_CRC4_MANUAL);

const code void * const code c_m_crc4_lines[] = {
    &line_c_m_crc4_tr,
    &line_c_m_crc4_mode,
#ifndef QUASAR_MINI
    &line_c_m_crc4_acrc,
#endif //QUASAR_MINI
    0
}; 
INST_MENU_LEVEL(c_m_crc4_menu,c_multiframe_menu_call,c_m_crc4_lines,
		"Configuration/", "Multiframe/CRC4", 0);


/* === /Configuration/Multiframing/CAS/ts16 ABCD XX menu === */

void abcd_printer (void) {
    unsigned char slotnr;
    unsigned char abcdoff;
    int val;

    slotnr = tsnum ();
    if ((0 == slotnr) || (16 == slotnr)) {
        PUTS_CODE ("Empty");
        return;
    }
    PRINTF ("Timeslot %d: ",  slotnr);

    abcdoff = slotnr/2;
    if (slotnr % 2) val = abcd.cdata[abcdoff] & 0x0F;
    else val = abcd.cdata[abcdoff] >> 4;
    PRINTF ("%x", val);
}

void abcd_handler (void) {
    unsigned char slotnr;
    unsigned char abcdoff;
    unsigned char abcd_new;

    slotnr = tsnum ();

    if ((0 == slotnr) || (16 == slotnr)) {
        return;
    }
    PUTS_CODE ("Enter ABCD code(hex):");

    abcd_new = get1bhex ();
    abcdoff = slotnr/2;
    if (slotnr % 2) {
        abcd.cdata[abcdoff] &= 0xF0;
	abcd.cdata[abcdoff] |= abcd_new & 0x0F;
    } else {
        abcd.cdata[abcdoff] &= 0x0F;
	abcd.cdata[abcdoff] |= abcd_new << 4;
    }
}

INST_MLCB(c_m_cas_abcd_group_line_1, 1, "", abcd_printer, abcd_handler);
INST_MLCB(c_m_cas_abcd_group_line_2, 2, "", abcd_printer, abcd_handler);
INST_MLCB(c_m_cas_abcd_group_line_3, 3, "", abcd_printer, abcd_handler);
INST_MLCB(c_m_cas_abcd_group_line_4, 4, "", abcd_printer, abcd_handler);
INST_MLCB(c_m_cas_abcd_group_line_5, 5, "", abcd_printer, abcd_handler);
INST_MLCB(c_m_cas_abcd_group_line_6, 6, "", abcd_printer, abcd_handler);
INST_MLCB(c_m_cas_abcd_group_line_7, 7, "", abcd_printer, abcd_handler);
INST_MLCB(c_m_cas_abcd_group_line_8, 8, "", abcd_printer, abcd_handler);
const code void * const code c_m_cas_abcd_group_lines[] = {
    &c_m_cas_abcd_group_line_1,
    &c_m_cas_abcd_group_line_2,
    &c_m_cas_abcd_group_line_3,
    &c_m_cas_abcd_group_line_4,
    &c_m_cas_abcd_group_line_5,
    &c_m_cas_abcd_group_line_6,
    &c_m_cas_abcd_group_line_7,
    &c_m_cas_abcd_group_line_8,
    0
};

INST_MENU_LEVEL(c_m_cas_abcd_1_7_menu, c_m_cas_menu_call, c_m_cas_abcd_group_lines,
    "Configuration/Multiframe/CAS/ts16 ABCD (", "1-7)", 0);
INST_MENU_LEVEL(c_m_cas_abcd_8_15_menu, c_m_cas_menu_call, c_m_cas_abcd_group_lines,
    "Configuration/Multiframe/CAS/ts16 ABCD (", "8-15)", 0);
INST_MENU_LEVEL(c_m_cas_abcd_16_23_menu, c_m_cas_menu_call, c_m_cas_abcd_group_lines,
    "Configuration/Multiframe/CAS/ts16 ABCD (", "16-23)", 0);
INST_MENU_LEVEL(c_m_cas_abcd_24_31_menu, c_m_cas_menu_call, c_m_cas_abcd_group_lines,
    "Configuration/Multiframe/CAS/ts16 ABCD (", "24-31)", 0);

unsigned char tsnum (void) {
    unsigned char off=0;


    if (curr_menu == &c_m_cas_abcd_1_7_menu) {
    } else if (curr_menu == &c_m_cas_abcd_8_15_menu) {
        off = 8;
    } else if (curr_menu == &c_m_cas_abcd_16_23_menu) {
        off = 16;
    } else { //c_m_cas_abcd_24_31_menu
        off = 24;
    }
    off += (line->flags_and_no & NMASK)-1;
    return off;
}


/* === /Configuration/Multiframing/CAS menu === */
void y_printer (void) {
    print_tristate (auto_off_on, cfg.cfg2, CFG2_RCAS_ON, CFG2_RCAS_MANUAL, 2);
}

void y_handler (void) {
    toggle_tristate (&(cfg.cfg2), CFG2_RCAS_ON, CFG2_RCAS_MANUAL, 2);
}

#ifndef QUASAR_MINI
INST_MLCB(line_c_m_y, 1, "Remote CAS Alarm: ", y_printer, y_handler);
#else //QUASAR_MINI
INST_MLT(line_c_m_y, 1, "Remote CAS Alarm: ", off_on,
	 &(cfg.cfg2), CFG2_RCAS_ON);
#endif //QUASAR_MINI
INST_MLT(line_c_m_cas_x1, 2, "X1", off_on,
	 &(cfg.cfg2), CFG2_X1);
INST_MLT(line_c_m_cas_x2, 3, "X2", off_on,
	 &(cfg.cfg2), CFG2_X2);
INST_MLT(line_c_m_cas_x3, 4, "X3", off_on,
	 &(cfg.cfg2), CFG2_X3);
INST_MLS(line_c_m_cas_abcd_1_7, 5, "ts16 ABCD (1-7)", &c_m_cas_abcd_1_7_menu);
INST_MLS(line_c_m_cas_abcd_8_15, 6, "ts16 ABCD (8-15)", &c_m_cas_abcd_8_15_menu);
INST_MLS(line_c_m_cas_abcd_16_23, 7, "ts16 ABCD (16-23)", &c_m_cas_abcd_16_23_menu);
INST_MLS(line_c_m_cas_abcd_24_31, 8, "ts16 ABCD (24-31)", &c_m_cas_abcd_24_31_menu);
const code void * const code c_m_cas_lines[] = {
    &line_c_m_y,
    &line_c_m_cas_x1,
    &line_c_m_cas_x2,
    &line_c_m_cas_x3,
    &line_c_m_cas_abcd_1_7,
    &line_c_m_cas_abcd_8_15,
    &line_c_m_cas_abcd_16_23,
    &line_c_m_cas_abcd_24_31,
    0
}; 
INST_MENU_LEVEL(c_m_cas_menu,c_multiframe_menu_call,c_m_cas_lines,
		"Configuration/", "Multiframe/CAS", 0);
void c_m_cas_menu_call (void) {
    curr_menu = &c_m_cas_menu;
}


/* === /Configuration/Framing/Multiframe menu === */
INST_MLS(line_c_m_cas, 1, "CAS", &c_m_cas_menu);
INST_MLS(line_c_m_crc4, 2, "CRC4", &c_m_crc4_menu);
INST_MLT(line_c_m_casonoff, 3, "CAS", on_off,
	 &(cfg.cfg0), CFG0_CAS);
const code void * const code c_multiframe_lines[] = {
    &line_c_m_cas,
    &line_c_m_crc4,
    &line_c_m_casonoff,
    0
};
INST_MENU_LEVEL(c_multiframe_menu,config_menu_call,c_multiframe_lines,
		"Configuration/", "Multiframe", 0);
void c_multiframe_menu_call (void) {
    curr_menu = &c_multiframe_menu;
}

/* === /Configuration/Framing/(Inter)National bits menu === */
INST_MLT(line_c_f_bits_sa4, 1, "Sa4", off_on,
	 &(cfg.cfg1), CFG1_NAT0);
INST_MLT(line_c_f_bits_sa5, 2, "Sa5", off_on,
	 &(cfg.cfg1), CFG1_NAT1);
INST_MLT(line_c_f_bits_sa6, 3, "Sa6", off_on,
	 &(cfg.cfg1), CFG1_NAT2);
INST_MLT(line_c_f_bits_sa7, 4, "Sa7", off_on,
	 &(cfg.cfg1), CFG1_NAT3);
INST_MLT(line_c_f_bits_sa8, 5, "Sa8", off_on,
	 &(cfg.cfg1), CFG1_NAT4);
INST_MLT(line_c_f_bits_sifas, 6, "Si0", off_on,
	 &(cfg.cfg1), CFG1_INAT0);
// Si not containing FAS
INST_MLT(line_c_f_bits_sinfas, 7, "Si1", off_on,
	 &(cfg.cfg1), CFG1_INAT1);
const code void * const code c_f_bits_lines[] = {
    &line_c_f_bits_sa4,
    &line_c_f_bits_sa5,
    &line_c_f_bits_sa6,
    &line_c_f_bits_sa7,
    &line_c_f_bits_sa8,
#ifndef QUASAR_MINI
    &line_c_f_bits_sifas,
    &line_c_f_bits_sinfas,
#endif //QUASAR_MINI
    0
};
INST_MENU_LEVEL(c_f_bits_menu,c_framing_menu_call,c_f_bits_lines,
		"Configuration/", "Framing/(Inter)National bits", 0);
void c_f_bits_menu_call (void) {
    curr_menu = &c_f_bits_menu;
}

/* === /Configuration/Framing menu === */
void rai_printer (void) {
    print_tristate (auto_off_on, cfg.cfg2, CFG2_RAI_ON, CFG2_RAI_MANUAL, 2);
}
void rai_handler (void) {
    toggle_tristate (&(cfg.cfg2), CFG2_RAI_ON, CFG2_RAI_MANUAL, 2);
}
INST_MLS(line_c_f_bits, 1, "(Inter)National bits", &c_f_bits_menu);
INST_MLT(line_c_f_rcv_tr, 2, "Receive", framing_transparent,
	 &(cfg.cfg0), CFG0_RCV_TR);
INST_MLT(line_c_f_xmit_tr, 3, "Xmit", framing_transparent,
	 &(cfg.cfg0), CFG0_XMIT_TR);
#ifndef QUASAR_MINI
INST_MLCB(line_c_f_rai, 4, "RAI: ", rai_printer, rai_handler);
#else //QUASAR_MINI
INST_MLT(line_c_f_rai, 4, "RAI", off_on,
	 &(cfg.cfg2), CFG2_RAI_ON);
#endif //QUASAR_MINI
const code void * const code c_framing_lines[] = {
    &line_c_f_bits,
    &line_c_f_rcv_tr,
    &line_c_f_xmit_tr,
    &line_c_f_rai,
    0
};
INST_MENU_LEVEL(c_framing_menu,config_menu_call,c_framing_lines,
		"Configuration/", "Framing", 0);
void c_framing_menu_call (void) {
    curr_menu = &c_framing_menu;
}

/* === /Configuration menu === */
INST_MLS(line_c_framing, 1, "Framing", &c_framing_menu);
INST_MLS(line_c_multiframe, 2, "Multiframe", &c_multiframe_menu);
INST_MLT(line_c_linecode, 3, "Line code", ami_hdb3,
	 &(cfg.cfg0), CFG0_RCV_HDB3);
INST_MLT(line_c_master, 4, "Clock source", line_int,
	 &(cfg.cfg0), CFG0_MASTER);
INST_MLCB(line_c_factory, 7, "Factory", 0, ecfg_factory_cfg);
INST_MLCB(line_c_restore, 8, "Restore", 0, ecfg_restore_cfg);
INST_MLCB(line_c_save, 9, "Save", 0, ecfg_save_cfg);
const code void * const code config_lines[] = {
    &line_c_framing,
    &line_c_multiframe,
    &line_c_linecode,
    &line_c_master,
    &line_c_factory,
    &line_c_restore,
    &line_c_save,
    0
};
INST_MENU_LEVEL(config_menu,main_menu_call,config_lines,"Configuration", "", 0);
void config_menu_call (void) {
    curr_menu = &config_menu;
}
 
/* === /Status menu === */
INST_MLT(line_s_reset, 1, "Reset", empty_empty,
	 &(status.mgmt), 0x1);
const code void * const code status_lines[] = {
    &line_s_reset,
    0
};
INST_MENU_LEVEL(status_menu,main_menu_call,status_lines,"Status", "", 0);

/* === Main(/) menu === */
INST_MLS (line_conf,   1, "Configuration", &config_menu);
INST_MLS (line_status, 2, "Status", &status_menu);
INST_MLS (line_test,   3, "Test", &test_menu);
const void * const main_lines [] = {
    &line_conf,
    &line_status,
    &line_test,
    0
};

INST_MENU_LEVEL (main_menu, 0, main_lines, "", "", 0);
void main_menu_call (void) {
    curr_menu = &main_menu;
}
