#ifndef __PMC_E1_IF
#define __PMC_E1_IF

// Protocol version (should I use $Rev\$ from SVN ?)
const char *ecfg_proto_version = "$Rev: 134 $";

/*Include this file into your project,
 if you want to work with ioctl call for ptmc_e1.o module.*/
//Define ioctl commands
//ioctl related section
/*Define device's magic number*/
#define PTMC_E1_8_MAGIC 0xa3
/**************************************************************/
#define PTMC_E1_RESET _IO(PTMC_E1_8_MAGIC, 2)
/**************************************************************/
#define PTMC_E1_READ_BAR0 _IOWR(PTMC_E1_8_MAGIC, 3, struct pmc_data_pass)
//read register in BAR0
//pass pointer to structure pmc_data_pass
/**************************************************************/
#define PTMC_E1_WRITE_BAR0 _IOW(PTMC_E1_8_MAGIC, 4, struct pmc_data_pass)
//write register in BAR0
//pass pointer to structure pmc_data_pass
/**************************************************************/
#define PTMC_E1_READ_BAR1 _IOWR(PTMC_E1_8_MAGIC, 5, struct pmc_data_pass)
//read register in BAR1
//pass pointer to structure pmc_data_pass
/**************************************************************/
#define PTMC_E1_WRITE_BAR1 _IOW(PTMC_E1_8_MAGIC, 6, struct pmc_data_pass)
//write register in BAR1
//pass pointer to structure pmc_data_pass
/**************************************************************/
#define PTMC_E1_GET_PCI_SLOT _IOR(PTMC_E1_8_MAGIC, 7, unsigned long)
//read device's pci slot number
//pass pointer to unsigned long
/**************************************************************/
#define PTMC_E1_GET_PCI_BUSNUM _IOR(PTMC_E1_8_MAGIC, 8, unsigned long)
//read device's pci bus number
//pass pointer to unsigned long
/**************************************************************/
#define PTMC_E1_CHAN_CONFIGURE _IOWR(PTMC_E1_8_MAGIC, 9, struct pmc_config_chan)
//configure framer.
//pass pointer to structure pmc_config_chan.
/***************************************************************/
#define PTMC_E1_BOARD_CONFIGURE _IOWR(PTMC_E1_8_MAGIC, 10, struct pmc_config_board)
//configure board registers
//pass pointer to structure pmc_config_board
/***************************************************************/
#define PTMC_E1_ERR_EMU _IOW(PTMC_E1_8_MAGIC, 11, struct pmc_err_ins)
//use to perfome error emulation.
//pass pointer to structure pmc_err_ins
/***************************************************************/
#define PTMC_E1_GET_STATS _IOWR(PTMC_E1_8_MAGIC, 12, struct pmc_status_chan)
//get statistics from framer
//pass pointer to structure pmc_status_chan, corresponding members of structure should be feeled with zeros.
/**Comment: If force to reset counters, return values which were in counters before reset*/
/***************************************************************/
#define PTMC_E1_CAS_XMIT_PASS _IOWR(PTMC_E1_8_MAGIC, 13, struct pmc_cas_xmit_data)
//get version
/***************************************************************/
#define PTMC_E1_GET_VERSION _IOR(PTMC_E1_8_MAGIC, 14, struct pmc_version)
//use to change cas data manually in test perposes
//pass pointer to pmc_cas_xmit_data structure
//#define PTMC_E1_PASS_ABCD_ZAP_FAKE _IOW(PTMC_E1_8_MAGIC, 15, unsigned int) //0xYYYXXXXXABCD, YYY - span number, XXXXX - channel number

#define PTMC_E1_8_MCN 14 //max cmd number.

#define PMC_BIT(a) (1<<a)

/** If not described: 0 - Disable, 1 - Enable */

/** Major configuration */
enum cfg0_t {
  CFG0_CAS = PMC_BIT(0),     /** CAS  (1=CSS/0=CAS)   1-14-5*/
  CFG0_RCV_TR = PMC_BIT(1),  /** Transparent on receive 1=transparent/0=framing on 1-12-3*/
  CFG0_XMIT_TR = PMC_BIT(2), /** Transparent on transmit 1-12-2*/
  CFG0_MASTER = PMC_BIT(3),  /** Master/Slave = 1/0*/
  CFG0_RCV_HDB3 = PMC_BIT(4),  /**Enable HDB encoding on RX 1-15-7*/
  CFG0_XMIT_HDB3 = PMC_BIT(5), /**Enable HDB encoding on TX 1-15-5*/
};

/** International/National bits */
enum cfg1_t {
  CFG1_NAT0 = PMC_BIT(0),   /**Sa4 1-17-4 */
  CFG1_NAT1 = PMC_BIT(1),   /**Sa5 1-17-3 */
  CFG1_NAT2 = PMC_BIT(2),   /**Sa6 1-17-2 */
  CFG1_NAT3 = PMC_BIT(3),   /**Sa7 1-17-1 */
  CFG1_NAT4 = PMC_BIT(4),   /**Sa8 1-17-0 */
  CFG1_INAT0 = PMC_BIT(5),  /**TIU0, FAS frame 1-12-0*/
  CFG1_INAT1 = PMC_BIT(6),  /**TIU1, NFAS frame 1-12-1*/
};

/** Misc indication bits*/
enum cfg2_t {
    CFG2_RAI_MANUAL = PMC_BIT(0),  /** 0=Auto/1=Manual  1-10-4*/
    CFG2_RAI_ON = PMC_BIT(1),      /** RAI: 0=normal/1=transmit alarm, ignored in Auto mode 1-16-5*/
    CFG2_RCAS_MANUAL = PMC_BIT(2), /** Remote CAS: 0=Auto/1=Manual 1-10-3*/
    CFG2_RCAS_ON = PMC_BIT(3),     /** Enable remote CAS (Y bit): 0=normal/1=LOM, ignored in Auto mode 1-13-2*/
    CFG2_X1 = PMC_BIT(4),           /** X1 multiframe 1-13-3*/
    CFG2_X2 = PMC_BIT(5),           /** X2 multiframe 1-13-1*/
    CFG2_X3 = PMC_BIT(6),           /** X3 multiframe 1-13-0*/
};


/**CRC4 related */
enum cfg3_t {
    CFG3_CRC4_TR = PMC_BIT(0), /** CRC4 transparent regeneration on 1-10-6*/
    CFG3_CRC4_DIS = PMC_BIT(1),    /** CRC4 1=disable/0=enable  1-10-2*/
    CFG3_CRC4_MANUAL = PMC_BIT(2), /**CRC4 Automatic CRC internetworking 0=on/1=off 1-10-5*/
};

/** 4 testing */
enum cfg4_t {
  CFG4_AIS = PMC_BIT(0),      /** TxAO 1=disable/0=enable 1-11-4*/
  CFG4_LLOOP = PMC_BIT(1),    /** Metallic loopback (shleif)  1=loop/0=normal 1-15-6*/
  CFG4_RLOOP = PMC_BIT(2),    /** Remote loopback (echo) 1=loop/0=normal  1-15-2*/
  CFG4_TX_DIS = PMC_BIT(3),   /** 1=Disable transmition on the board level, use to emulate LOS, transmition enabled by default*/
  CFG4_PLOOP = PMC_BIT(4),    /** Payload loopback 1=loop/0=normal 1-15-0 */
};




struct pmc_config_chan {
	unsigned char chan_num;		 /**chan_num & direction - chanal number [0,7] - 0x8x - only read data, 0x0x - write then read*/
	unsigned char cfg0;
	unsigned char cfg1;
	unsigned char cfg2;
	unsigned char cfg3;
	unsigned char cfg4;
};

/**Driver automaticaly reset corresponding bits to zero before execution*/
enum err_emu0_t {
    ERR_EMU0_LOF_NFAS = PMC_BIT(0), /** Emulate single frame violation (1=generate error)  1-19-4*/
    ERR_EMU0_LOF_FAS = PMC_BIT(1),  /** Emulate single frame violation (1=generate error) 1-19-5*/
    ERR_EMU0_CRC4 = PMC_BIT(2),     /** Emulate single CRC4 error (1=generate error)  1-19-6*/
};

struct pmc_err_ins {
	unsigned char chan_num;	    /** chan_num - chanal number [0,7];*/
	unsigned char pmc_err;
};

/**Misc status bits*/
enum stat0 {
  STAT0_LOS = PMC_BIT(0),       /** LOS signal 1-loss/0-ok 3-11-4*/
  STAT0_LOF = PMC_BIT(1),       /** LOF signal 1-loss/0-ok 3-10-7*/
  STAT0_CAS_LOM = PMC_BIT(2),   /** CAS LOM signal 1-loss/0-ok 3-10-6*/
  STAT0_CRC4_SYNC = PMC_BIT(3), /** CRC4 syncronisation 1-loss/0-ok 3-10-5*/
  STAT0_E_BIT = PMC_BIT(4),     /** bit E state 1=error/0=ok !(3-10-3 &  3-10-4)*/
  STAT0_CRC4 = PMC_BIT(5),      /** CRC4 state  1=error/0=ok  (3-11-6 | 3-11-7)*/
  STAT0_AIS = PMC_BIT(6),       /** AIS state 1=valid AIS or all ones/0=valid AIS is not being received 3-11-2*/
  STAT0_RAIS = PMC_BIT(7),      /** RAIS state 1=error/0=normal 3-11-1*/
};

/** International/National bits */
enum stat1  {
	STAT1_NAT0 = PMC_BIT(0),      /**Sa4 3-18-4 */
	STAT1_NAT1 = PMC_BIT(1),      /**Sa5 3-18-3 */
	STAT1_NAT2 = PMC_BIT(2),      /**Sa6 3-18-2 */
	STAT1_NAT3 = PMC_BIT(3),      /**Sa7 3-18-1 */
	STAT1_NAT4 = PMC_BIT(4),      /**Sa8 3-18-0 */
	STAT1_INAT0 = PMC_BIT(5),     /**3-15-7*/
	STAT1_INAT1 = PMC_BIT(6),     /**3-18-7*/
	STAT1_SLIP = PMC_BIT(7),      /**Slip occured  1=Slip, auto clearing after reading 4-12-1*/
};

/**Multifreim X Y bits*/
enum stat2 {
	STAT2_Y = PMC_BIT(0),
	STAT2_X1 = PMC_BIT(1),
	STAT2_X2 = PMC_BIT(2),
        STAT2_X3 = PMC_BIT(3),
};

/** Overrun bits */
enum ovr_mgmt_t {
	OVR_HDB3 = PMC_BIT(0),
	OVR_FAS = PMC_BIT(1),
	OVR_CRC4 = PMC_BIT(2),
};

struct pmc_status_chan {
	unsigned char chan_num;  /**chan_num - channel number [0,7];*/
	unsigned char stat0;
	unsigned char stat1;
	unsigned char stat2;
	unsigned char mgmt;     /** Counters managment (clear counters) set 0x01 to clear HDB3 counter, 0x02 - clear FAS, 0x04 - clear CRC4*/
	unsigned char ovr;      /** Counters overrun bits 0x01 - CRC4, 0x02 - FAS, 0x04 - HDB3*/
	unsigned short hdb3;     /** HDB3 counter, if forced to reset counter according to mgmt content, returns value which were in counter before reset, the same for other counters*/
	unsigned short crc4;     /** CRC4 counter*/
	unsigned short fas;      /** FAS counter*/
        unsigned char  cas_data[16]; /**High nibble of cas_data[0] = cas nibble for 0 channel, low nibble of cas_data[0] = cas nibble for 1 channel.... */
};

struct pmc_data_pass
{
	unsigned long raddress; /** Register address */
	unsigned long rdata;    /** Register data */
};


/** Selection of mode & a/b select*/
enum mode1 {
    MODE1_AB = PMC_BIT(0), /**0=use A signals/1=use B signals*/
    MODE1_NF1_GEN_ENA =  PMC_BIT(1), /**1=Enable to generate Netref1*/
    MODE1_NF2_GEN_ENA =  PMC_BIT(2), /**1=Enable to generate Netref2*/
    MODE1_NF1_USE_ENA =  PMC_BIT(3), /**1=Enable to use  Netref1 for syncronization*/
    MODE1_NF2_USE_ENA =  PMC_BIT(4), /**1=Enable to use  Netref2 for syncronization*/
    MODE1_MA_SL_SEL = PMC_BIT(5),  /**1=Master on CT-BUS*/
};

/**Structure to configure PLL*/
struct pmc_config_board
{
	unsigned char mode1; /** Selection of mode & a/b select*/
	//unsigned char sync_line_source; /** - 0000-0111 - sync with corresponding e1 chanal, 1000 - sync with Netref1, 1001 - sync with Netref2.*/
};

enum direct {
    DIRECT_IN = PMC_BIT(7), /**1=only read data to know what we are transmiting, 0 = rewrite transmiting data*/
};

struct pmc_cas_xmit_data
{
    unsigned char chan_num; /**channel number and direction - 0x8x - only read data, 0x0x - write then read back*/
    unsigned char cdata[16];  /**The same as in pmc_status_chan.cas_data[]*/
};

struct pmc_version
{
    char proto_version[16];
    char driver_version[16];
    unsigned long hw_fw_rev; /** Hardware/Firmware revision **/
};

#endif //__PMC_E1_IF
